#include <avr/io.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "types.h"
#include "fat.h"
#include "sd.h"

#define BYTES_PER_SECTOR 512

u32 FatFirstDataSector;
u16 FatSectorsPerCluster;
u32 FirstFATSector;
u32 FatRootCluster;

u32 selectedDirectoryCluster;
u32 selectedFileCluster, selectedFileSize;
u32 fileCluster, fileSize, fileSector;
u08 fileSectorsReaded;
u16 fileBytesReaded;
u08* fileCachePtr;

u08 sectorCache[BYTES_PER_SECTOR];
struct direntry *de;


u32 clust2sect(u32 clust)
{
	return ((clust-2) * FatSectorsPerCluster) + FatFirstDataSector;
}

//
// read partition data and setup constants.
//
u08 FAT_init()
{
	struct partrecord *pr;
	struct bpb710 *bpb;

    u08 *SectorBuffer = sectorCache;
    
    fileSector = 0;


    // read partition table
    SD_ReadSector(0, SectorBuffer);

	// get the partition record
	pr = (struct partrecord *) ((struct partsector *) SectorBuffer)->psPart;

	// and find the first valid sector
	FatFirstDataSector = pr->prStartLBA;

	// Read the Partition BootSector
    SD_ReadSector(FatFirstDataSector, SectorBuffer);

	// get BIOS parameter block
	bpb = (struct bpb710 *)  ((struct bootsector710 *) SectorBuffer)->bsBPB;

	// and setup some constants
	FirstFATSector     			= bpb->bpbResSectors + FatFirstDataSector;
	FatFirstDataSector 	        += bpb->bpbResSectors + bpb->bpbFATs * bpb->bpbBigFATsecs;     //zde zacinaji clustery.
	FatSectorsPerCluster        = bpb->bpbSecPerClust;
	FatRootCluster				= bpb->bpbRootClust;
	selectedDirectoryCluster	= FatRootCluster;

	free(SectorBuffer);

	return 0;
}

//
// find next cluster in the FAT chain
//
u32 get_next_cluster(u32 cluster)
{
	u32 FatOffset = cluster << 2; //4 byty na adrasaci clusteru
	u32 sector    = FirstFATSector + (FatOffset / BYTES_PER_SECTOR);
	u16 offset    = FatOffset % BYTES_PER_SECTOR;

        SD_ReadSector(sector, sectorCache);

	FatOffset =  (*((u32*) &(sectorCache)[offset])) & FAT32_MASK;

	if ( FatOffset >= (CLUST_EOFS & FAT32_MASK) )
		FatOffset = 0;

	return FatOffset;
}

//
// read next sector from opened file
//
void FAT_read_sector(u08* buff)
{
	if(fileSectorsReaded >= FatSectorsPerCluster)
	{
		fileSector = clust2sect(fileCluster = get_next_cluster(fileCluster));
		fileSectorsReaded = 0;
	}

        SD_ReadSector(fileSector++, buff);
        fileSectorsReaded++;
	//fileBytesReaded = 0;
}

//
// read len bytes from opened file
//
u16 FAT_fread(u08* buf, u16 len)
{
	u16 i;
	if(len > fileSize) len = fileSize;
	fileSize -= len;
	u16 n = len;

	while(n)
	{
		if(!fileBytesReaded)
		{
			FAT_read_sector(sectorCache);
			fileCachePtr = sectorCache;
			fileBytesReaded = BYTES_PER_SECTOR;
		}

		if(n > fileBytesReaded) i = fileBytesReaded;
		else			  		i = n;

		fileBytesReaded -= i;
		n -= i;

		i++;
		while(--i) *buf++ = *fileCachePtr++;
	}

	return len;
}

//
// read one byte from opened file
//
s16 FAT_getc()
{	
	if(!fileSize) return EOF;
	fileSize--;

	if(!fileBytesReaded)
	{
		FAT_read_sector(sectorCache);
		fileCachePtr = sectorCache;
		fileBytesReaded = BYTES_PER_SECTOR;
	}
	fileBytesReaded--;

	return *fileCachePtr++;
}

//
// open selected file
//
void FAT_open_file()
{
	fileSector = clust2sect(fileCluster = selectedFileCluster);
	fileSize = selectedFileSize;
	fileSectorsReaded = 0;
	fileBytesReaded = 0;
}

//
//select item while listing
//
FAT_select_item()
{
    u32 cluster = (((u32)de->deHighClust) << 16) | de->deStartCluster;
	if(!cluster) cluster = FatRootCluster;

	if(de->deAttributes == ATTR_DIRECTORY) selectedDirectoryCluster = cluster;
	else
    {
          selectedFileSize = de->deFileSize;
          selectedFileCluster = cluster;
    }
}

u32 FAT_get_file_size()
{
        return fileSize;
}


void FAT_list_directory(u08 (*listCallbackFnc)(char* NameBuffer, BYTE params))
{
    u16 i,index;

    u08 NameBuffer[16];
    u08 *SectorBuffer = sectorCache;

    u32 cluster = selectedDirectoryCluster;
    u32 sector = clust2sect(cluster);
    u08 numsectors = 0;

	// always clear the longname buffer
	*NameBuffer = 0;

	index = 16; //crank it up
	do
	{
		if (index >= 16)
		{
			if(numsectors >= FatSectorsPerCluster)
            {
                  numsectors = 0;
                  sector = clust2sect(cluster = get_next_cluster(cluster));
            }

			SD_ReadSector(sector++, SectorBuffer);
			numsectors++;

			de = (struct direntry *) SectorBuffer;
			index = 0;
		}

		if (*de->deName != 0xE5)   // if not a deleted entry
		{
			if ((de->deAttributes & (ATTR_HIDDEN|ATTR_SYSTEM|ATTR_VOLUME)) == 0) // normal file entry (including directories)
			{
				if (!(de->deName[0] == '.' && de->deName[1] != '.')) //dot - sam na sebe?
				{
					//if (*NameBuffer == 0)   //pokud predchozi entry neobsahoval dlouhej nazev
					{
						char *p = NameBuffer;
						for (i=0;i<8;i++)	*p++ = de->deName[i];		// copy first part
						while(*--p == ' ');
						p++;
					  	*p++ = '.';
						for (i=0;i<3;i++)	*p++ = (de->deExtension[i]);	// copy second part	(in lower case) maly pismena, pripona souboru
						while(*--p == ' ');
						if(*p != '.') p++;
						*p = 0;
					}

					if((*listCallbackFnc)(NameBuffer, de->deAttributes)) goto end;

				}
			}				
		}
		de++;		// point to next directory entry
		index++;	// increment index count within this sector
		
	}
	while (index == 16 || *de->deName);	// dokud neni posledni zapis nebo neni treba nacist novej cluster 0 in de->deName[0] if no more entries


end:
        //restore the original file cache
        if(fileSector) SD_ReadSector(fileSector - 1, sectorCache);
	return;
}



