#include <avr/io.h>
#include "types.h"
#include "sd.h"
#include "spi.h"


//sd card initialize
void SD_Init(void)
{
 DDRD |= 0x01;
	SD_CS_Deassert();
}

//sd send command
uint8 SD_SendCommand(uint8 cmd, uint32 arg)
{
	uint8 r1;
	uint8 retry=0;
	

	for(r1=0;r1<6;r1++)	SPI_transfer_byte(0xff);  //TODO: musi to tu bejt?

	SD_CS_Assert();
	
	SPI_transfer_byte(cmd | 0x40);	//send command
	SPI_transfer_byte(arg>>24);
	SPI_transfer_byte(arg>>16);
	SPI_transfer_byte(arg>>8);
	SPI_transfer_byte(arg);
	SPI_transfer_byte(0x95);
	
	retry = 100;
	while((r1 = SPI_transfer_byte(0xff)) == 0xff)	//wait response
		if(!--retry) break;			//time out error

	SD_CS_Deassert();

	return r1;				//return state
}

//reset sd card (software)
uint8 SD_Reset(void)
{
	uint8 i;
	uint16 retry=0;
	uint8 r1=0;

	SPI_normal_speed();
	SPI_set_speed(0);
	SPI_disable_IRQ();

	retry = 1000;

	do
	{
		for(i=0;i<100;i++) SPI_transfer_byte(0xff);
		r1 = SD_SendCommand(0, 0);	//send idle command
		if(!--retry) return 1;		//time out
	} 
    while(r1 != 0x01);

	retry = 20000;
	do
	{
		r1 = SD_SendCommand(1, 0);	//send active command
		if(!--retry) return 2;	//time out
	} 
    while(r1);

	SPI_double_speed();
	r1 = SD_SendCommand(59, 0);	//disable CRC

	r1 = SD_SendCommand(16, 512);	//set sector size to 512
	return 0;	//normal return
}


//read one sector
uint8 SD_ReadSector(uint32 sector, uint8* buffer)
{
	uint8 r1;
	uint16 i, retry=0;

	SPI_set_speed(0);
	SPI_disable_IRQ();
	SPI_double_speed();

	retry = 100;

	do
	{
		r1 = SD_SendCommand(17, sector<<9);	//read command
		if(!--retry) return 1;		//time out
	}
    while(r1 != 0x00);


	SD_CS_Assert();

        //wait to start recieve data
	retry=10000;
	while(SPI_transfer_byte(0xff) != 0xfe)
		if(!--retry)
		{
			SD_CS_Deassert();
			return 2;			//time out
		}

    i=513;
    while(--i) *buffer++ = SPI_transfer_byte(0xff);

	SPI_transfer_byte(0xff);//crc
	SPI_transfer_byte(0xff);
	SD_CS_Deassert();

	return 0;
}









/*

//wirite one sector //not used in this application
uint8 MMC_SD_WriteSector(uint32 sector, uint8* buffer)
{
	uint8 r1;
	uint16 i;
	uint8 retry=0;

	do
	{
		r1 = MMC_SD_SendCommand(24, sector<<9);	//send command
		retry++;
		if(retry>10) return 1;	//time out
	} 
    while(r1 != 0x00);


	SD_CS_Assert();

	SPI_transfer_byte(0xff);
	SPI_transfer_byte(0xff);
	SPI_transfer_byte(0xff);
	SPI_transfer_byte(0xff);
	SPI_transfer_byte(0xff);
	SPI_transfer_byte(0xff);
	SPI_transfer_byte(0xfe);	//send start byte

	for(i=0; i<512; i++)		//send 512 bytes data
		SPI_transfer_byte(*buffer++);

	SPI_transfer_byte(0xff);
	SPI_transfer_byte(0xff);
	r1 = SPI_transfer_byte(0xff);

	if( (r1&0x1f) != 0x05)	//judge if it successful
	{
		SD_CS_Deassert();
		return r1;
	}
	
        //wait no busy
	while(!SPI_transfer_byte(0xff));//if(retry++ > 50){SD_CS_Deassert();return 1;}

	SD_CS_Deassert();

	return 0;
}

uint32 MMC_SD_ReadCapacity()
{
	uint8 r1;
	uint16 i;
	uint16 temp;
	uint8 buffer[16];
	uint32 Capacity;
	//uint8 retry=0;

	r1 = MMC_SD_SendCommand(9, 0);	//send command  //READ CSD
	if(r1 != 0x00)
		return r1;

	SD_CS_Assert();
	while(SPI_transfer_byte(0xff) != 0xfe);
	
	for(i=0;i<16;i++)
		buffer[i]=SPI_transfer_byte(0xff);


	SPI_transfer_byte(0xff);
	SPI_transfer_byte(0xff);
	SPI_transfer_byte(0xff);
	
	SD_CS_Deassert();

//*********************************
//	C_SIZE
	i = buffer[6]&0x03;
	i<<=8;
	i += buffer[7];
	i<<=2;
	i += ((buffer[8]&0xc0)>>6);

//*********************************
//  C_SIZE_MULT

	r1 = buffer[9]&0x03;
	r1<<=2;
	r1 += ((buffer[10]&0x80)>>7);


//**********************************
// BLOCKNR

	r1+=2;

	temp = 1;
	while(r1)
	{
		temp*=2;
		r1--;
	}
	
	Capacity = ((uint32)(i+1))*((uint32)temp);

/////////////////////////
// READ_BL_LEN

	i = buffer[6]&0x0f;

//*************************
//BLOCK_LEN

	temp = 1;
	while(i)
	{
		temp*=2;
		i--;
	}
//************************


//************** formula of the capacity ******************
//
//  memory capacity = BLOCKNR * BLOCK_LEN
//	
//	BLOCKNR = (C_SIZE + 1)* MULT
//
//           C_SIZE_MULT+2
//	MULT = 2
//
//               READ_BL_LEN
//	BLOCK_LEN = 2
//**********************************************

//The final result
	
	Capacity *= (uint32)temp;	 
	return Capacity;		
}

*/
